#include <stdlib.h>
#include <stdio.h>
#include <sys/shm.h>
#include <time.h>
#include <unistd.h>
#include "shared.h"

int GetNextFreeSlot(SharedStruct* sharedMem);
void StartProducer(SharedStruct* sharedMem);
void Produce(SharedStruct* sharedMem);
int produce_item(void);
int insert_item(int item, SharedStruct* sharedMem);

// Checks for SharedID, then begins producing numbers into shared memory.
int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s SharedID# \n", argv[0]);
        exit(1);
    }
    srandom((unsigned int) time(NULL));
    int sharedID = StringToNumber(argv[1]);
    SharedStruct* sharedMem = shmat(sharedID, NULL, 0);
    StartProducer(sharedMem);
    return 0;
}

// Checks for the next available slot in buffer and returns slot number.
// TODO:    Fix function to get next free slot and return slot number.
int GetNextFreeSlot(SharedStruct* sharedMem)
{
    int slot = -1;
    for (int i = 0; i < 10; i++)
    {
        if ((sharedMem->buffer[i] != -1) && (sharedMem->buffer[(i+1)%10] == -1))
        {
            slot = (i+1)%10;
            return slot;
        }
    }
    return slot;
}

// Starts the producer process.
void StartProducer(SharedStruct* sharedMem)
{
    //  TODO: For testing purposes, revert max to 100
    for (int i = 0; i < 5; i++)
    {
        sem_wait(&sharedMem->empty);
        Produce(sharedMem);
        sem_post(&sharedMem->full);
    }
}

// Produces and puts a number into the next free slot in the buffer.
void Produce(SharedStruct* sharedMem)
{
    int insertNum;
    sem_wait(&sharedMem->mutex);
    insertNum = produce_item();
    insert_item(insertNum, sharedMem);
    printf("%d is inserted.\n", insertNum);
    sem_post(&sharedMem->mutex);
    return;
}

// Generate a number between 0 and 9.
int produce_item(void)
{
    return random() % 10;
}

// Insert a number into the shared memory.
// TODO:    Reimplement insertion using the next slot from GetNextFreeSlot().
int insert_item(int item, SharedStruct* sharedMem)
{
    sharedMem->buffer[0] = item;
    return 1;
}